#!/usr/bin/perl

use Net::NNTP;
use Getopt::Long;
use FileHandle;
use Time::Local;

my $file = "";
my $nntp_server = "";
my $port;
my $message_id;
my $sender;
my %fields;

my @headres;
my @body;

sub process_file {

	$fh = FileHandle->new($file, "r");
	if(!defined($fh)){
		print "Error: Unable to open file $file for reading\n";
		exit(0);
	}

	while(<$fh>){
		
		chomp;
		if(/^\s*$/){
			last;
		}
		($header, $val) = split(/:/);
		$header =~ s/^\s*//; $header =~ s/\s*$//;
		$val =~ s/^\s*//; $val =~ s/\s*$//;

		if($header =~ /^name$/i){
			$fields{"name"} = $val;				
			$sender = $val;
			next;
		}
		if($header =~ /^from$/i){
			$fields{"from"} = $val;				
			if(defined($sender) && !($sender eq "")){
				push(@headers, "From: $sender <$val>\n");
			}
			else{
				push(@headers, "From: $val\n");
			}
			next;
		}
		if($header =~ /^date$/i){
			$fields{"date"} = $val;
			#Convert the date into FRC 822 date format
			($day, $month, $year) = split(/\//, $val);
			if(!defined($day) || !defined($month) || !defined($year)){
				print ("Invalid date format : $val\n");
				exit(0);
			}
			$nntp_date_str = `date -R -d $month/$day/$year`; chomp $nntp_date_str;
			push(@headers, "Date: $nntp_date_str\n");
			next;
		}
		if($header =~ /^ng$/i){
			$fields{"ng"} = $val;
			push(@headers, "Newsgroups: $val\n");
			next;
		}
		if($header =~ /^sub$/i){
			$fields{"subject"} = $val;
			push(@headers, "Subject: $val\n");
			next;
		}
		if($header =~ /^replyto$/i){
			$fields{"replyto"} = $val;
			push(@headers, "Reply-To: $val\n");
			next;
		}
		if($header =~ /^sender$/i){
			$fields{"sender"} = $val;
			push(@headers, "Sender: $val\n");
			next;
		}
		if($header =~ /^exp$/i){
			$fields{"expires"} = $val;
			#Convert the date into FRC 822 date format
			($day, $month, $year) = split(/\//, $val);
			if(!defined($day) || !defined($month) || !defined($year)){
				print ("Invalid date format : $val\n");
				exit(0);
			}
			$nntp_date_str = `date -R -d $month/$day/$year`; chomp $nntp_date_str;
			push(@headers, "Expires: $nntp_date_str\n");
			next;
		}
		print "Invaid syntax in input line : $_ \n";
		exit(0);
	}

	#Check all compulsory parameters

	if(!defined($fields{"date"})){
		print "Compulsory field Date missing in input file";
		exit(0);
	}
	if(!defined($fields{"subject"})){
		print "Compulsory field Subject missing in input file";
		exit(0);
	}
	if(!defined($fields{"ng"})){
		print "Compulsory field NG missing in input file";
		exit(0);
	}
	if(!defined($fields{"from"})){
		print "Compulsory field From missing in input file";
		exit(0);
	}
	if(!defined($fields{"name"})){
		print "Compulsory field Name missing in input file";
		exit(0);
	}
	push(@headers, "Message-ID: $message_id\n");

	#push(@headers, "Body: \n");
	push(@body, "\n");

	while(<$fh>){
		chomp;
		push(@body, "$_\n");
	}

	$dashes = "-----------------------------------------------------------------\n";
	push(@body, $dashes);
	push(@body, "$sender\n");
	
	$date = `date`;
	chomp($date);
	push(@body, "$date\n");

	$fortune = `/usr/games/fortune`;
	chomp $fortune;
	push(@body, "$fortune\n");

	push(@body, $dashes);
}

sub post_data {
	
	$nntp = Net::NNTP->new($nntp_server);
	if(defined($nntp)){
		if($nntp->post((@headers, @body))){
			print "Message successfully posted\n";
		}
		else{
			print "Error posting the message\n";
			print $nntp->message();
		}
		$nntp->quit;
	}
	else{
		print "Error: Unable to contact the news server: $nntp_server\n";
	}
}

sub main {

	$result = GetOptions(
	          "file=s" => \$file,
			  "server=s" => \$nntp_server,
	          );

	if(!$result){
		exit(0);
	}

	if($file =~ /^\s*$/){
		print "Please specify a filename which contains data to post\n";
		print "Usage: news.pl {-file data-file} {-server nntp-server}\n";
		exit(0);
	}
	if($nntp_server =~ /^\s*$/){
		print "Please specify a news server\n";
		print "Usage: news.pl {-file data-file} {-server nntp-server}\n";
		exit(0);
	}

	$user = $ENV{"USER"};
	$random = `echo \$RANDOM`; chomp $random;
	$this_host = `hostname`; chomp $this_host;
	$message_id = "<$user$random$this_host\@$nntp_server>";

	process_file($file);
	post_data();

}

main();
1;
